import logging
from logging import NullHandler

logging.getLogger(__name__).addHandler(NullHandler())

try:
    from ._version import __version__
except ImportError:
    __version__ = "0.0.0"

DEFAULT_HOST = "https://auth.app.kognic.com"
DEFAULT_TOKEN_ENDPOINT_RELPATH = "/v1/auth/oauth/token"



import json
import os
from dataclasses import dataclass
from typing import Optional, Union

REQUIRED_CREDENTIALS_FILE_KEYS = [
    "clientId",
    "clientSecret",
    "email",
    "userId",
    "issuer",
]


@dataclass
class ApiCredentials:
    client_id: str
    client_secret: str
    email: str
    user_id: int
    issuer: str


def parse_credentials(path: Union[str, dict]):
    if isinstance(path, dict):
        credentials = path
    else:
        try:
            with open(path) as f:
                credentials = json.load(f)
        except FileNotFoundError:
            raise FileNotFoundError(f"Could not find Api Credentials file at {path}") from None

    if not isinstance(credentials, dict):
        raise AttributeError(f"Could not json dict from {path}")

    for k in REQUIRED_CREDENTIALS_FILE_KEYS:
        if k not in credentials:
            raise KeyError(f"Missing key {k} in credentials file")

    return ApiCredentials(
        client_id=credentials.get("clientId"),
        client_secret=credentials.get("clientSecret"),
        email=credentials.get("email"),
        user_id=credentials.get("userId"),
        issuer=credentials.get("issuer"),
    )


def get_credentials(auth):
    if isinstance(auth, str):
        if auth.endswith(".json"):
            return parse_credentials(auth)
        raise ValueError("Bad auth credentials file, must be json")
    elif isinstance(auth, ApiCredentials):
        return auth
    else:
        raise ValueError("Bad auth credentials, must be path to credentials file, or ApiCredentials object")


def get_credentials_from_env():
    creds = os.getenv("KOGNIC_CREDENTIALS")
    if creds:
        client_credentials = parse_credentials(creds)
        return client_credentials.client_id, client_credentials.client_secret

    client_id = os.getenv("KOGNIC_CLIENT_ID")
    client_secret = os.getenv("KOGNIC_CLIENT_SECRET")

    if client_id and client_secret:
        return client_id, client_secret

    # fallbacks
    creds = os.getenv("ANNOTELL_CREDENTIALS")
    if creds:
        client_credentials = parse_credentials(creds)
        return client_credentials.client_id, client_credentials.client_secret

    client_id = os.getenv("ANNOTELL_CLIENT_ID")
    client_secret = os.getenv("ANNOTELL_CLIENT_SECRET")

    return client_id, client_secret


def resolve_credentials(auth=None, client_id: Optional[str] = None, client_secret: Optional[str] = None):
    has_credentials_tuple = client_id is not None and client_secret is not None
    if auth is not None:
        if has_credentials_tuple:
            raise ValueError("Choose either auth or client_id+client_secret")
        if isinstance(auth, tuple):
            if len(auth) != 2:
                raise ValueError("Credentials tuple must be tuple of (client_id, client_secret)")
            client_id, client_secret = auth
        else:
            creds = get_credentials(auth)
            client_id = creds.client_id
            client_secret = creds.client_secret
    elif not has_credentials_tuple:
        client_id, client_secret = get_credentials_from_env()

    return client_id, client_secret


if __name__ == "__main__":
    credentials = get_credentials_from_env()
    print(credentials)



import datetime
import unittest

from kognic.auth.base.auth_client import AuthClient


class TestAuthClient(AuthClient):
    def __init__(self, token: dict):
        self._token = token

    @property
    def token(self):
        return self._token


class AuthClientTests(unittest.TestCase):
    def test_auth_client(self):
        expires_at = 1729670977
        expect = datetime.datetime(2024, 10, 23, 8, 9, 37, tzinfo=datetime.timezone.utc)
        client = TestAuthClient(token={"expires_at": expires_at, "expires_in": 3600})
        self.assertEqual(client.expires_at, expect)



import logging
from asyncio import Lock
from typing import Optional

import httpx
from authlib.integrations.httpx_client import AsyncOAuth2Client
from authlib.oauth2.rfc6749 import OAuth2Token

from kognic.auth import DEFAULT_HOST, DEFAULT_TOKEN_ENDPOINT_RELPATH
from kognic.auth.base.auth_client import AuthClient
from kognic.auth.credentials_parser import resolve_credentials

log = logging.getLogger(__name__)


class _AsyncFixedClient(AsyncOAuth2Client):
    async def _refresh_token(self, url, **kwargs):
        try:
            return await super(_AsyncFixedClient, self)._refresh_token(url, **kwargs)
        except httpx.HTTPStatusError as e:
            if e.response.status_code == 401:
                log.info("Refresh token expired, resetting auth session")
                return await self.fetch_token()
            raise


class HttpxAuthAsyncClient(AuthClient):
    def __init__(
        self,
        *,
        auth=None,
        client_id: Optional[str] = None,
        client_secret: Optional[str] = None,
        host: str = DEFAULT_HOST,
        token_endpoint: str = DEFAULT_TOKEN_ENDPOINT_RELPATH,
        **kwargs,
    ):
        """
        There is a variety of ways to set up the authentication.
        :param auth: authentication credentials
        :param client_id: client id for authentication
        :param client_secret: client secret for authentication
        :param host: base url for authentication server
        :param token_endpoint: relative path to the token endpoint
        :param kwargs: additional params to pass into Httpx Client Constructor
        """
        self.host = host
        self.token_url = f"{host}{token_endpoint}"

        client_id, client_secret = resolve_credentials(auth, client_id, client_secret)

        self._oauth_client = _AsyncFixedClient(
            client_id=client_id,
            client_secret=client_secret,
            update_token=self._update_token,
            token_endpoint=self.token_url,
            grant_type="client_credentials",
            **kwargs,
        )
        self._oauth_client.register_compliance_hook("access_token_response", AuthClient.check_rate_limit)
        self._oauth_client.register_compliance_hook("refresh_token_response", AuthClient.check_rate_limit)

        self._lock = Lock()

    @property
    def token(self):
        return self._oauth_client.token

    async def _update_token(self, token: OAuth2Token, refresh_token=None, access_token=None):
        self._log_new_token()

    @property
    async def session(self) -> AsyncOAuth2Client:
        if not self.token:
            async with self._lock:
                # check again when coming out of the lock that the token is still not set
                if not self.token:
                    token = await self._oauth_client.fetch_token()
                    await self._update_token(token)
        return self._oauth_client

    async def close(self):
        await self._oauth_client.aclose()



import base64
import json
import logging
from datetime import datetime, timezone
from typing import Optional

log = logging.getLogger(__name__)


class AuthClient:
    def _log_new_token(self):
        if "expires_in" in self.token:
            log.info(f"Got new token, with ttl={self.token['expires_in']} and expires {self.expires_at}")
        else:
            log.warning(f"Got new token that is likely not valid: missing expires_in but got {self.token.keys()}")

    @property
    def access_token(self):
        return self.token["access_token"] if self.token else None

    @property
    def claims(self) -> Optional[dict]:
        """
        For introspection, no validation is done.
        :return:
        """
        if self.token:
            return json.loads(base64.b64decode(self.access_token.split(".")[1] + "=="))

    @property
    def expires_at(self):
        return datetime.fromtimestamp(self.token["expires_at"], tz=timezone.utc) if self.token else None

    @property
    def token(self):
        raise NotImplementedError

    @staticmethod
    def check_rate_limit(response):
        if response.status_code == 429:
            log.error("Client authentication rate limit exceeded! Please slow down.")
        return response



import logging
import threading
from typing import Optional

import requests
from authlib.common.errors import AuthlibBaseError
from authlib.integrations.requests_client import OAuth2Session

from kognic.auth import DEFAULT_HOST, DEFAULT_TOKEN_ENDPOINT_RELPATH
from kognic.auth.base.auth_client import AuthClient
from kognic.auth.credentials_parser import resolve_credentials

log = logging.getLogger(__name__)


class _FixedSession(OAuth2Session):
    def refresh_token(self, url, **kwargs):
        try:
            super(_FixedSession, self).refresh_token(url, **kwargs)
        except AuthlibBaseError as e:
            if e.error == "invalid_token":
                log.info("Refresh token expired, resetting auth session")
                return self.fetch_token()
            raise
        except requests.exceptions.HTTPError as e:
            # with authlib >= 1.0.0
            if e.response.status_code == 401 and "invalid_token" == e.response.json().get("error"):
                log.info("Refresh token expired, resetting auth session")
                return self.fetch_token()
            raise


# https://docs.authlib.org/en/latest/client/oauth2.html
class RequestsAuthSession(AuthClient):
    """
    Not thread safe
    """

    def __init__(
        self,
        *,
        auth=None,
        client_id: Optional[str] = None,
        client_secret: Optional[str] = None,
        host: str = DEFAULT_HOST,
        token_endpoint: str = DEFAULT_TOKEN_ENDPOINT_RELPATH,
        **kwargs,
    ):
        """
        There is a variety of ways to set up the authentication.
        :param auth: authentication credentials
        :param client_id: client id for authentication
        :param client_secret: client secret for authentication
        :param host: base url for authentication server
        :param token_endpoint: relative path to the token endpoint
        :param kwargs: additional params to pass into Client Constructor
        """
        self.host = host
        self.token_url = f"{host}{token_endpoint}"

        client_id, client_secret = resolve_credentials(auth, client_id, client_secret)

        self.oauth_session = _FixedSession(
            client_id=client_id,
            client_secret=client_secret,
            token_endpoint_auth_method="client_secret_post",
            update_token=self._update_token,
            token_endpoint=self.token_url,
            **kwargs,
        )
        self.oauth_session.register_compliance_hook("access_token_response", AuthClient.check_rate_limit)
        self.oauth_session.register_compliance_hook("refresh_token_response", AuthClient.check_rate_limit)

        self._lock = threading.RLock()

    @property
    def token(self):
        return self.oauth_session.token

    def _update_token(self, token, access_token=None, refresh_token=None):
        self._log_new_token()

    @property
    def session(self):
        if not self.token:
            with self._lock:
                if not self.token:
                    # check again when coming out of the lock that the token is still not set
                    token = self.oauth_session.fetch_access_token(url=self.token_url)
                    self._update_token(token)
        return self.oauth_session.session
